/*
 *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
 *
 *
 *  Copyright (C) 2002 Rochet Sylvain
 *
 *  gtkatlantic is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>

#include "global.h"
#include "callback.h"
#include "client.h"
#include "game.h"
#include "interface.h"

#include "engine.h"


/* **** Show Configuration Panel */
void Callback_ShowConfiguration(GtkWidget* menuitem)  {

	interface_create_config_panel();
}


/* **** Show all monopd servers */
void Callback_ShowMetaServer(GtkWidget* button)  {

	create_connection_metaserver(CONNECT_TYPE_META_GETLIST);
}


/* **** Get games from all monopd server */
void Callback_RefreshGameMetaServer(GtkWidget* button)  {

	create_connection_metaserver(CONNECT_TYPE_META_GETGAME);
}


/* **** Get games from another server */
void Callback_GetgamesButton(GtkWidget* button, GtkWidget **data)  {

	guint32 port;
	gchar *host;

	host = g_strstrip( g_strdup( gtk_entry_get_text( GTK_ENTRY(data[0]) ) ) );
	port = atol( gtk_entry_get_text( GTK_ENTRY(data[1]) ) );

	create_connection_get_games(host, port);
	g_free(host);
}


/* get games: event "row_select" */
void Callback_GetGame_Select(GtkWidget* clist, gint row)  {

	gchar **ptrtxt, *texts[10];
	guint32 i;


	if(global->mode_gameclist != GAMECLIST_MODE_GAMELIST)  return;

	ptrtxt = g_malloc(8192);

	for(i = 0 ; i < 8 ; i++)  {

		memset(ptrtxt, 0, 8192);
		gtk_clist_get_text(GTK_CLIST(clist), row, i, ptrtxt);
		texts[i] = g_strdup(*ptrtxt);
	}
	g_free(ptrtxt);

	if(global->selected_host)  g_free(global->selected_host);
	if(global->selected_gametype)  g_free(global->selected_gametype);

	global->selected_host = g_strdup( texts[0] );
	global->selected_port = atoi( texts[5] );
	global->selected_gametype = g_strdup( texts[6] );
	global->selected_gameid = atoi( texts[7] );

	for(i = 0 ; i < 8 ; i++)
		g_free(texts[i]);
}


/* get games: event "row_unselect" */
void Callback_GetGame_Unselect(GtkWidget* clist, gint row)  {

	if(global->selected_host)  g_free(global->selected_host);
	if(global->selected_gametype)  g_free(global->selected_gametype);
	global->selected_host = 0;
	global->selected_port = 0;
	global->selected_gametype = 0;
	global->selected_gameid = 0;
}


/* **** Join/create a game */
void Callback_ConnectButton(GtkWidget* clist)  {

	guint16 connectid;
	gchar *text;

	if(!global->selected_host) return;
	if(!global->selected_port) return;
	if(!global->selected_gametype) return;
	if(!global->selected_gameid) return;

	if(! client_connect(&connectid, global->selected_host, global->selected_port) )   return;
	connection[connectid]->type = CONNECT_TYPE_MONOPD_GAME;

	/* set nickname */
	text = g_strdup_printf(".n%s\n", config->nickname);
	client_send(connectid, text, strlen(text));
	g_free(text);

	game_alloc();
	game->connectid = connectid;

	/* create a game */
	if(global->selected_gameid < 0)  {

		game->type = GAME_TYPE_CREATE;
		text = g_strdup_printf(".gn%s\n", global->selected_gametype);
		client_send(connectid, text, strlen(text));
		g_free(text);
	}

	/* join a game */
	if(global->selected_gameid > 0)  {

		game->type = GAME_TYPE_JOIN;
		text = g_strdup_printf(".gj%d\n", global->selected_gameid);
		client_send(connectid, text, strlen(text));
		g_free(text);
	}

	g_free(global->selected_host);
	g_free(global->selected_gametype);
	global->selected_host = 0;
	global->selected_port = 0;
	global->selected_gametype = 0;
	global->selected_gameid = 0;
}


/* **** Start the game :) */
void Callback_StartButton(GtkWidget* button)  {

	client_send(game->connectid, ".gs\n", 4);
}


/* **** Leave the game */
void Callback_LeaveButton(GtkWidget* button)  {

	client_disconnect(game->connectid);
	g_free(game);
	interface_create_getgamespage();
}


/* **** Enter in the real program */
void Callback_EnterButton(GtkWidget* button)  {

	frame_destroy(global->logo_frame);
	g_free(data->pngfile_logo_image);
	g_free(data->pngfile_logo_alpha);

	interface_create_getgamespage();
}


/* *** send data to server */
void Callback_SendButton(GtkWidget* Button, GtkWidget **Data) {

	guchar *tmp, *tmp2;

	tmp = g_strdup( gtk_entry_get_text( GTK_ENTRY(Data[0]) ) );
	g_strstrip(tmp);

	tmp2 = g_strconcat(tmp, "\n", NULL);
	g_free(tmp);
	tmp = tmp2;

	client_send(game->connectid, tmp, strlen(tmp) );
	g_free(tmp);

	gtk_entry_set_text(GTK_ENTRY(Data[0]), "");
}


/* *** Un/Float Server Message Box*/
void CallBack_toggle_float_message(GtkWidget* widget) {

	if(global->phase != PHASE_GAMEPLAY) return;

	game->message_float = !game->message_float;
	interface_message_and_chat_float_unfloat();
}


/* *** Un/Float Chat Box*/
void CallBack_toggle_float_chat(GtkWidget* widget) {

	if(global->phase != PHASE_GAMEPLAY) return;

	game->chat_float = !game->chat_float;
	interface_message_and_chat_float_unfloat();
}


/* *** "delete_event" signal window message */
void CallBack_delete_window_message(GtkWidget* widget) {

	gtk_widget_hide( GTK_WIDGET(global->MessageBox) );
	gtk_widget_reparent(GTK_WIDGET(global->MessageBox), global->Trick);
	gtk_widget_destroy(widget);
	widget = 0;
	game->message_float = 0;
	interface_message_and_chat_float_unfloat();
}


/* *** "delete_event" signal window chat */
void CallBack_delete_window_chat(GtkWidget* widget) {

	gtk_widget_hide( GTK_WIDGET(game->ChatBox) );
	gtk_widget_reparent(GTK_WIDGET(game->ChatBox), global->Trick);
	gtk_widget_destroy(widget);
	widget = 0;
	game->chat_float = 0;
	interface_message_and_chat_float_unfloat();
}


/* *** "destroy" signal window message */
void CallBack_destroy_window_message(GtkWidget* widget) {

}


/* *** "destroy" signal window chat */
void CallBack_destroy_window_chat(GtkWidget* widget) {

}


/* *** save and apply the config */
void CallBack_ok_button_configwindow(GtkWidget* widget)  {

	guchar *filename, *errormsg;
	FILE *file;

	filename = g_strconcat(global->path_home, "config.xml", NULL);

	file = fopen(filename, "wt");
	if(!file)  {

		errormsg = g_strdup_printf("[ERROR] Can't write config file at \"%s\"", filename);
		text_insert_message(errormsg, strlen(errormsg));
		g_free(errormsg);
		g_free(filename);
		return;
	}
	g_free(filename);

	fprintf(file, "<?xml version=\"1.0\"?>\n<gtkatlantic>\n");
	fprintf(file, "\t<nick name=\"%s\" />\n",
		gtk_entry_get_text( GTK_ENTRY( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_player_nickname") ) )  );
	fprintf(file, "\t<metaserver2 host=\"%s\" port=\"%d\" autogetgames=\"%d\" autogetlist=\"%d\" />\n",
		gtk_entry_get_text( GTK_ENTRY( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_metaserver_host") ) ),
		atoi( gtk_entry_get_text( GTK_ENTRY( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_metaserver_port") ) ) ),
		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_metaserver_autogetgames") ) ),
		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_metaserver_autogetlist") ) )  );
	fprintf(file, "\t<getgames host=\"%s\" port=\"%d\" autogetgames=\"%d\" />\n",
		gtk_entry_get_text( GTK_ENTRY( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_anotherserver_host") ) ),
		atoi( gtk_entry_get_text( GTK_ENTRY( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_anotherserver_port") ) ) ),
		gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_anotherserver_autogetgames") ) )  );
	fprintf(file, "\t<message max_lines=\"%d\" />\n",
		atoi( gtk_entry_get_text( GTK_ENTRY( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_buffersize_servermessages") ) ) ) );
	fprintf(file, "\t<chat max_lines=\"%d\" />\n",
		atoi( gtk_entry_get_text( GTK_ENTRY( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_buffersize_chat") ) ) ) );
	fprintf(file, "\t<game playerlist_position=\"%s\" />\n",
		gtk_entry_get_text( GTK_ENTRY( gtk_object_get_data(GTK_OBJECT(global->ConfigWin), "config_gameinterface_playerlistposition") ) ) );
	fprintf(file, "</gtkatlantic>\n");

	fclose(file);
	read_config_files();

	gtk_widget_destroy(global->ConfigWin);
}


/* *** button command pressed */
void CallBack_button_command_pressed(GtkWidget* button) {

	guchar *ptr = gtk_object_get_data(GTK_OBJECT(button), "command");

	client_send(game->connectid, ptr, strlen(ptr) );
}


/* *** button command pressed */
void Callback_AuctionButton(GtkWidget* button, GtkWidget* entry)  {

	guchar *tosend, *auction, *money;

	auction = gtk_object_get_data(GTK_OBJECT(entry), "auctionid");

	money = g_strdup( gtk_entry_get_text( GTK_ENTRY(entry) ) );
	g_strstrip(money);

	tosend = g_strconcat(".ab", auction, ":", money, "\n", NULL);

	client_send(game->connectid, tosend, strlen(tosend) );

	g_free(money);
	g_free(tosend);

	gtk_entry_set_text(GTK_ENTRY(entry), "");
}


/* *** callback clic on estate */
void Callback_ClicOnEstate(GtkWidget* widget, GdkEventButton *event)  {

	guint8 id = atoi( gtk_object_get_data(GTK_OBJECT(widget), "estateid") );
	guchar *text;

	if(event->button == 1)  { /* left clic -> build house */

		text = g_strdup_printf(".hb%d\n", id);
		client_send(game->connectid, text, strlen(text) );
		g_free(text);
	}
	if(event->button == 3)  { /* right clic -> sell house */

		text = g_strdup_printf(".hs%d\n", id);
		client_send(game->connectid, text, strlen(text) );
		free(text);
	}
	if(event->button == 2)  { /* middle clic -> toggle_mortgage */

		text = g_strdup_printf(".em%d\n", id);
		client_send(game->connectid, text, strlen(text) );
		g_free(text);
	}
}


/* *** callback enter an estate */
void Callback_EnterAnEstate(GtkWidget* widget)  {

	guint8 id = atoi( gtk_object_get_data(GTK_OBJECT(widget), "estateid") );

	guint16 frame = game->board_frame;
	guint16 pic = game->estate[id].pic;

	pic_set_bgcolor(frame, pic, 0x000000);
	pic_set_alpha(frame, pic, 170);
}


/* *** callback leave an estate */
void Callback_LeaveAnEstate(GtkWidget* widget)  {

	guint8 id = atoi( gtk_object_get_data(GTK_OBJECT(widget), "estateid") );

	guint16 frame = game->board_frame;
	guint16 pic = game->estate[id].pic;

	pic_unset_alpha(frame, pic);
}


/* *** callback for game option type bool */
void Callback_toggle_boolean_gameoption(GtkWidget* widget)  {

	guchar *text, *cmd;

	if(! widget)  return;

	if(game->i_am_the_master)  {

		cmd = gtk_object_get_data(GTK_OBJECT(widget), "command");
		text = g_strdup_printf("%s%d\n", cmd, gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget) )  );
		client_send(game->connectid, text, strlen(text) );
		g_free(text);
	}
}


/* *** callback for the entry menu change nickname */
void CallBack_menu_change_nickname(GtkWidget* widget)  {

	if(global->phase >= PHASE_GAMECREATE)
		interface_create_nicknamewin();
}


/* *** callback for the ok button of window change nickname */
void CallBack_ok_button_changenick_window(GtkWidget* widget, GtkWidget *Win)  {

	GtkWidget *Entry = gtk_object_get_data(GTK_OBJECT(Win), "nickname");
	guchar *nickname, *sendstr;

	nickname = g_strdup( gtk_entry_get_text(GTK_ENTRY( Entry ) )  );

	if(strncmp(game->player[ get_player_slot_with_playerid(game->my_playerid) ].name, nickname, strlen(nickname) ) )  {

		sendstr = g_strdup_printf(".n%s\n", nickname);
		client_send(game->connectid, sendstr, strlen(sendstr) );
		g_free(sendstr);
	}

	g_free(nickname);
	gtk_widget_destroy(Win);
}


/* *** callback for the entry menu change nickname */
void CallBack_menu_reconnect(GtkWidget* widget)  {

	guchar *host, *tmp, *cookie, *filename;
	guint32 port;
	FILE *file;

	guint16 connectid;
	gchar *text;

//	if(global->phase > PHASE_GETGAMES) return;


	/* read cookie file */
	filename = g_strconcat(global->path_home, "cookie", NULL);
	file = fopen(filename, "rt");
	g_free(filename);

	if(!file)  return;

	host = g_malloc0(4096);
	fscanf(file, "%s\n", host );

	tmp = g_malloc0(4096);
	fscanf(file, "%s\n", tmp );
	port = atoi(tmp);
	g_free(tmp);

	cookie = g_malloc0(4096);
	fscanf(file, "%s\n", cookie);

	fclose(file);


	/* rejoin game */
	if(! client_connect(&connectid, host, port) )   {

		g_free(host);
		g_free(cookie);
		return;
	}

	g_free(host);
	connection[connectid]->type = CONNECT_TYPE_MONOPD_GAME;

	game_alloc();
	game->connectid = connectid;

	/* send cookie */
	text = g_strdup_printf(".R%s\n", cookie);
	client_send(connectid, text, strlen(text));
	g_free(text);


	g_free(cookie);
}
